import pyarrow.parquet as pq
import pandas as pd
import json
from typing import List, Callable, Iterator, Union, Optional
from sportsdataverse.config import WNBA_BASE_URL, WNBA_TEAM_BOX_URL, WNBA_PLAYER_BOX_URL, WNBA_TEAM_SCHEDULE_URL
from sportsdataverse.errors import SeasonNotFoundError
from sportsdataverse.dl_utils import download

def espn_wnba_schedule(dates=None, season_type=None) -> pd.DataFrame:
    """espn_wnba_schedule - look up the WNBA schedule for a given season

    Args:
        dates (int): Used to define different seasons. 2002 is the earliest available season.
        season_type (int): 2 for regular season, 3 for post-season, 4 for off-season.

    Returns:
        pd.DataFrame: Pandas dataframe containing schedule dates for the requested season.
    """
    if dates is None:
        dates = ''
    else:
        dates = '&dates=' + str(dates)
    if season_type is None:
        season_type = ''
    else:
        season_type = '&seasontype=' + str(season_type)

    url = "http://site.api.espn.com/apis/site/v2/sports/basketball/wnba/scoreboard?limit=300{}{}".format(dates,season_type)
    ev = pd.DataFrame()
    resp = download(url=url)

    if resp is not None:
        events_txt = json.loads(resp)

        events = events_txt['events']
        for event in events:
            if 'links' in event['competitions'][0]['competitors'][0]['team'].keys():
                del event['competitions'][0]['competitors'][0]['team']['links']
            if 'links' in event['competitions'][0]['competitors'][1]['team'].keys():
                del event['competitions'][0]['competitors'][1]['team']['links']
            if event['competitions'][0]['competitors'][0]['homeAway']=='home':
                event['competitions'][0]['home'] = event['competitions'][0]['competitors'][0]['team']
            else:
                event['competitions'][0]['away'] = event['competitions'][0]['competitors'][0]['team']
            if event['competitions'][0]['competitors'][1]['homeAway']=='away':
                event['competitions'][0]['away'] = event['competitions'][0]['competitors'][1]['team']
            else:
                event['competitions'][0]['home'] = event['competitions'][0]['competitors'][1]['team']

            del_keys = ['broadcasts','geoBroadcasts', 'headlines']
            for k in del_keys:
                if k in event['competitions'][0].keys():
                    del event['competitions'][0][k]

            ev = ev.append(pd.json_normalize(event['competitions'][0]))
    ev = pd.DataFrame(ev)
    return ev

def espn_wnba_calendar(season=None) -> pd.DataFrame:
    """espn_wnba_calendar - look up the WNBA calendar for a given season

    Args:
        season (int): Used to define different seasons. 2002 is the earliest available season.

    Returns:
        pd.DataFrame: Pandas dataframe containing
        calendar dates for the requested season.

    Raises:
        ValueError: If `season` is less than 2002.
    """
    if int(season) < 2002:
        raise SeasonNotFoundError("season cannot be less than 2002")
    url = "http://site.api.espn.com/apis/site/v2/sports/basketball/wnba/scoreboard?dates={}".format(season)
    resp = download(url=url)
    txt = json.loads(resp)['leagues'][0]['calendar']
    datenum = list(map(lambda x: x[:10].replace("-",""),txt))
    date = list(map(lambda x: x[:10],txt))

    year = list(map(lambda x: x[:4],txt))
    month = list(map(lambda x: x[5:7],txt))
    day = list(map(lambda x: x[8:10],txt))

    data = {"season": season,
            "datetime" : txt,
            "date" : date,
            "year": year,
            "month": month,
            "day": day,
            "dateURL": datenum
    }
    df = pd.DataFrame(data)
    df['url']="http://site.api.espn.com/apis/site/v2/sports/basketball/wnba/scoreboard?dates="
    df['url']= df['url'] + df['dateURL']
    return df



SGITHUB = 'https://raw.githubusercontent.com/saiemgilani/'
NFLVERSEGITHUB='https://raw.githubusercontent.com/nflverse/'
CFB_BASE_URL = SGITHUB+'cfbfastR-data/master/pbp/parquet/play_by_play_{season}.parquet'
CFB_ROSTER_URL = SGITHUB+'cfbfastR-data/master/rosters/parquet/rosters_{season}.parquet'
CFB_TEAM_LOGO_URL  = SGITHUB+'cfbfastR-data/master/teams/teams_colors_logos.parquet'
CFB_TEAM_SCHEDULE_URL = SGITHUB+'cfbfastR-data/master/schedules/parquet/schedules_{season}.parquet'
CFB_TEAM_INFO_URL = SGITHUB+'cfbfastR-data/master/team_info/parquet/team_info_{season}.parquet'

NFL_BASE_URL = NFLVERSEGITHUB+'nflfastR-data/master/data/play_by_play_{season}.parquet'
NFL_PLAYER_STATS_URL = NFLVERSEGITHUB+'nflfastR-data/master/data/player_stats.parquet'
NFL_ROSTER_URL = NFLVERSEGITHUB+'nflfastR-roster/master/data/nflfastR-roster.csv.gz'
NFL_TEAM_LOGO_URL  = NFLVERSEGITHUB+'nflfastR-data/master/teams_colors_logos.csv'
NFL_TEAM_SCHEDULE_URL = SGITHUB+'nflfastR-data/master/schedules/sched_{season}.parquet'

NHL_BASE_URL = NFLVERSEGITHUB+'nflfastR-data/master/data/play_by_play_{season}.parquet'
NHL_PLAYER_STATS_URL = NFLVERSEGITHUB+'nflfastR-data/master/data/player_stats.parquet'
NHL_ROSTER_URL = NFLVERSEGITHUB+'nflfastR-roster/master/data/nflfastR-roster.csv.gz'
NHL_TEAM_LOGO_URL  = NFLVERSEGITHUB+'nflfastR-data/master/teams_colors_logos.csv'
NHL_TEAM_SCHEDULE_URL = SGITHUB+'nflfastR-data/master/schedules/sched_{season}.parquet'


MBB_BASE_URL = SGITHUB+'hoopR-data/master/mbb/pbp/parquet/play_by_play_{season}.parquet'
MBB_TEAM_BOX_URL = SGITHUB+'hoopR-data/master/mbb/team_box/parquet/team_box_{season}.parquet'
MBB_PLAYER_BOX_URL = SGITHUB+'hoopR-data/master/mbb/player_box/parquet/player_box_{season}.parquet'
MBB_TEAM_LOGO_URL  = SGITHUB+'hoopR-data/master/mbb/teams_colors_logos.csv'
MBB_TEAM_SCHEDULE_URL = SGITHUB+'hoopR-data/master/mbb/schedules/parquet/mbb_schedule_{season}.parquet'
NBA_BASE_URL = SGITHUB+'hoopR-data/master/nba/pbp/parquet/play_by_play_{season}.parquet'
NBA_TEAM_BOX_URL = SGITHUB+'hoopR-data/master/nba/team_box/parquet/team_box_{season}.parquet'
NBA_PLAYER_BOX_URL  = SGITHUB+'hoopR-data/master/nba/player_box/parquet/player_box_{season}.parquet'
NBA_TEAM_SCHEDULE_URL = SGITHUB+'hoopR-data/master/nba/schedules/parquet/nba_schedule_{season}.parquet'
WBB_BASE_URL = SGITHUB+'wehoop-data/master/wbb/pbp/parquet/play_by_play_{season}.parquet'
WBB_TEAM_BOX_URL = SGITHUB+'wehoop-data/master/wbb/team_box/parquet/team_box_{season}.parquet'
WBB_PLAYER_BOX_URL = SGITHUB+'wehoop-data/master/wbb/player_box/parquet/player_box_{season}.parquet'
WBB_TEAM_LOGO_URL  = SGITHUB+'wehoop-data/master/wbb/teams_colors_logos.csv'
WBB_TEAM_SCHEDULE_URL = SGITHUB+'wehoop-data/master/wbb/schedules/parquet/wbb_schedule_{season}.parquet'
WNBA_BASE_URL = SGITHUB+'wehoop-data/master/wnba/pbp/parquet/play_by_play_{season}.parquet'
WNBA_TEAM_BOX_URL = SGITHUB+'wehoop-data/master/wnba/team_box/parquet/team_box_{season}.parquet'
WNBA_PLAYER_BOX_URL  = SGITHUB+'wehoop-data/master/wnba/player_box/parquet/player_box_{season}.parquet'
WNBA_TEAM_SCHEDULE_URL = SGITHUB+'wehoop-data/master/wnba/schedules/parquet/wnba_schedule_{season}.parquet'




# Always prefer setuptools over distutils
# To use a consistent encoding
from codecs import open
from os import path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="sportsdataverse",
    project_name="sportsdataverse",
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version="0.0.10",
    description="Retrieve Sports data in Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # The project's main homepage.
    url="https://github.com/saiemgilani/sportsdataverse-py",
    project_urls={
        "Docs": "https://sportsdataverse-py.sportsdataverse.org/",
        "Bug Tracker": "https://github.com/saiemgilani/sportsdataverse-py/issues",
    },
    # Author details
    author="Saiem Gilani",
    author_email="saiem.gilani@gmail.com",
    # Maintainer
    maintainer="Saiem Gilani",
    # Choose your license
    license="MIT",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    # What does your project relate to?
    keywords="nfl college football data epa statistics web scraping",
    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=["tests", "tests.*"]),
    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],
    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        "numpy>=1.13.0",
        "pandas >= 1.0.3",
        "beautifulsoup4>=4.4.0",
        "requests>=2.18.1",
        "lxml>=4.2.1",
        "pyarrow>=1.0.1",
        "pygithub>=1.51",
        "scipy>=1.4.0",
        "matplotlib>=2.0.0",
        "tqdm>=4.50.0",
        "attrs>=20.3.0",
        "xgboost>=1.2.0",
    ],
    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        #    'dev': ['check-manifest'],
        "test": [
            "pytest>=6.0.2",
            "mypy>=0.782",
            "pytest-cov>=2.10.1",
            "pytest-xdist>=2.1.0",
        ],
    },
    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    include_package_data=True,
    package_data={
        "sportsdataverse": [
            "cfb/models/*",
            "nfl/models/*",
        ]
    },
    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    # entry_points={
    #    'console_scripts': [
    #        'sample=sample:main',
    #    ],
    # },
)
